#!/usr/bin/env python
from deepdive import *
import ddlib

@tsv_extractor
@returns(lambda
        p_id   = "text",
        loc_id   = "text",
        feature = "text",
    :[])
def extract(
        p_id          = "text",
        loc_id          = "text",
        p_begin_index = "int",
        p_end_index   = "int",
        loc_begin_index = "int",
        loc_end_index   = "int",
        doc_id         = "text",
        sent_index     = "text",
        tokens         = "text",
        lemmas         = "text",
        pos_tags       = "text",
        ner_tags       = "text",
    ):
    """
    Uses DDLIB to generate features for the any relation.
    """
    # Create a DDLIB sentence object, which is just a list of DDLIB Word objects
    sent = []
    ner_tags = ner_tags.split("~^~")
    pos_tags = pos_tags.split("~^~")
    lemmas = lemmas.split("~^~")
    tokens = tokens.split("~^~")
    for i,t in enumerate(tokens):
        sent.append(ddlib.Word(
            begin_char_offset=None,
            end_char_offset=None,
            word=t,
            lemma=lemmas[i],
            pos=pos_tags[i],
            ner=ner_tags[i],
            dep_par=-1,  # Note that as stored from CoreNLP 0 is ROOT, but for DDLIB -1 is ROOT
            dep_label=""))

    # Create DDLIB Spans for the two person mentions
    p1_span = ddlib.Span(begin_word_id=p_begin_index, length=(p_end_index-p_begin_index+1))
    p2_span = ddlib.Span(begin_word_id=loc_begin_index, length=(loc_end_index-loc_begin_index+1))

    # Generate the generic features using DDLIB
    for feature in ddlib.get_generic_features_relation(sent, p1_span, p2_span):
        yield [p_id, loc_id, feature]
