#!/usr/bin/env python
from deepdive import *
import random
from collections import namedtuple

NationalityLabel = namedtuple('NationalityLabel', 'p_id, loc_id, label, type')

@tsv_extractor
@returns(lambda
        p_id   = "text",
        loc_id   = "text",
        label   = "int",
        rule_id = "text",
    :[])
# heuristic rules for finding positive/negative examples of spouse relationship mentions
def supervise(
        p_id="text", p_begin="int", p_end="int",
        loc_id="text", loc_begin="int", loc_end="int",
        doc_id="text", sentence_index="text", sentence_text="text",
        tokens="text", lemmas="text", pos_tags="text", ner_tags="text",
    ):
    # Constants
    ORIGIN = frozenset(["born", "from"])
    MAX_DIST = 10

    # Common data objects
    p_end_idx = min(p_end, loc_end)
    loc_start_idx = max(p_begin, loc_begin)
    loc_end_idx = max(p_end,loc_end)
    intermediate_lemmas = lemmas[p_end_idx+1:loc_start_idx]
    intermediate_ner_tags = ner_tags[p_end_idx+1:loc_start_idx]
    tail_lemmas = lemmas[loc_end_idx+1:]
    nat = NationalityLabel(p_id=p_id, loc_id=loc_id, label=None, type=None)

    # Rule: Candidates that are too far apart
    if len(intermediate_lemmas) > MAX_DIST:
        yield nat._replace(label=-1, type='neg:far_apart')

    # Rule: Candidates that have a third person in between
    if 'PERSON' in intermediate_ner_tags:
        yield nat._replace(label=-1, type='neg:third_person_between')

    if len(ORIGIN.intersection(intermediate_lemmas)) > 0:
        yield nat._replace(label=1, type='pos:from/born')
    # Rule: of/between
    if ("of" in intermediate_lemmas) and (len(intermediate_lemmas)):
        yield nat._replace(label=1, type='pos:of_between')
